﻿function GetPluginSettings()
{
	return {
		"name":			"Painter Tools",				// as appears in 'insert object' dialog, can be changed as long as "id" stays the same
		"id":			"PainterTools",				// this is used to identify this plugin and is saved to the project; never change it
		"version":		"1.0",					// (float in x.y format) Plugin version - C2 shows compatibility warnings based on this
		"description":	"Let the user draw on screen.",
		"author":		"Kozalo",
		"help url":		"http://kozalo.tk/paintertools",
		"dependency":	"FileSaver.min.js",
		"category":		"General",				// Prefer to re-use existing categories, but you can set anything here
		"type":			"world",				// either "world" (appears in layout and is drawn), else "object"
		"rotatable":	false,					// only used when "type" is "world".  Enables an angle property on the object.
		"flags":		0						// uncomment lines to enable flags...
					//	| pf_singleglobal		// exists project-wide, e.g. mouse, keyboard.  "type" must be "object".
					//	| pf_texture			// object has a single texture (e.g. tiled background)
						| pf_position_aces		// compare/set/get x, y...
						| pf_size_aces			// compare/set/get width, height...
					//	| pf_angle_aces			// compare/set/get angle (recommended that "rotatable" be set to true)
					//	| pf_appearance_aces	// compare/set/get visible, opacity...
					//	| pf_tiling				// adjusts image editor features to better suit tiled images (e.g. tiled background)
					//	| pf_animations			// enables the animations system.  See 'Sprite' for usage
					//	| pf_zorder_aces		// move to top, bottom, layer...
						| pf_nosize				// prevent resizing in the editor
					//	| pf_effects			// allow WebGL shader effects to be added
					//  | pf_predraw			// set for any plugin which draws and is not a sprite (i.e. does not simply draw
												// a single non-tiling image the size of the object) - required for effects to work properly
	};
};

////////////////////////////////////////
// Parameter types:
// AddNumberParam(label, description [, initial_string = "0"])			// a number
// AddStringParam(label, description [, initial_string = "\"\""])		// a string
// AddAnyTypeParam(label, description [, initial_string = "0"])			// accepts either a number or string
// AddCmpParam(label, description)										// combo with equal, not equal, less, etc.
// AddComboParamOption(text)											// (repeat before "AddComboParam" to add combo items)
// AddComboParam(label, description [, initial_selection = 0])			// a dropdown list parameter
// AddObjectParam(label, description)									// a button to click and pick an object type
// AddLayerParam(label, description)									// accepts either a layer number or name (string)
// AddLayoutParam(label, description)									// a dropdown list with all project layouts
// AddKeybParam(label, description)										// a button to click and press a key (returns a VK)
// AddAnimationParam(label, description)								// a string intended to specify an animation name
// AddAudioFileParam(label, description)								// a dropdown list with all imported project audio files

////////////////////////////////////////
// Conditions

// AddCondition(id,					// any positive integer to uniquely identify this condition
//				flags,				// (see docs) cf_none, cf_trigger, cf_fake_trigger, cf_static, cf_not_invertible,
//									// cf_deprecated, cf_incompatible_with_triggers, cf_looping
//				list_name,			// appears in event wizard list
//				category,			// category in event wizard list
//				display_str,		// as appears in event sheet - use {0}, {1} for parameters and also <b></b>, <i></i>
//				description,		// appears in event wizard dialog when selected
//				script_name);		// corresponding runtime function name
				
// example				
//AddNumberParam("Number", "Enter a number to test if positive.");
//AddCondition(0, cf_none, "Is number positive", "My category", "{0} is positive", "Description for my condition!", "MyCondition");

AddCondition(0, cf_none, "Is changed", "Painter Tools", "IsChanged", "Triggered when one of parameters is changed.", "IsChanged");
AddCondition(1, cf_none, "Is drawing", "Painter Tools", "Is drawing", "Returns true when the user is drawing a figure..", "IsDrawing");
AddCondition(2, cf_none, "Is saved", "Painter Tools", "Is saved", "Returns true after the user has saved an image.", "IsSaved");
AddCondition(3, cf_none, "Is loaded", "Painter Tools", "Is loaded", "Returns true after the user has loaded an image.", "IsLoaded");

////////////////////////////////////////
// Actions

// AddAction(id,				// any positive integer to uniquely identify this action
//			 flags,				// (see docs) af_none, af_deprecated
//			 list_name,			// appears in event wizard list
//			 category,			// category in event wizard list
//			 display_str,		// as appears in event sheet - use {0}, {1} for parameters and also <b></b>, <i></i>
//			 description,		// appears in event wizard dialog when selected
//			 script_name);		// corresponding runtime function name

// example
//AddStringParam("Message", "Enter a string to alert.");
//AddAction(0, af_none, "Alert", "My category", "Alert {0}", "Description for my action!", "MyAction");

AddComboParamOption("Invisible");
AddComboParamOption("Visible");
AddComboParam("Visibility", "Choose whether to hide or show the toolbar.");
AddAction(0, af_none, "Set visible", "Appearance", "Set <b>{0}</b>", "Hide or show the toolbar.", "SetVisible");

AddComboParamOption("Pencil");
AddComboParamOption("Line");
AddComboParamOption("Rectangle");
AddComboParamOption("Circle");
AddComboParamOption("Arrow");
AddComboParamOption("Text");
AddComboParamOption("Note");
AddComboParam("Tool", "The tool.", 0);
AddAction(1, af_none, "Set current tool", "", "Set current tool.", "Set current tool.", "SetTool");

AddNumberParam("Width", "The width of lines.", "1.0");
AddAction(2, af_none, "Set width of lines", "", "Set the width of lines.", "Set the width of lines.", "SetLinesWidth");

AddStringParam("Color of lines", "The color of lines.", '"#000000"');
AddAction(3, af_none, "Set color of lines", "", "Set the color of lines.", "Set the color of lines.", "SetLinesColor");

AddStringParam("Color of background", "The color of the background.", '"#ffffff"');
AddAction(4, af_none, "Set color of the background", "", "Set the color of the background.", "Set the color of the background.", "SetBackgroundColor");

AddNumberParam("Set whether fill shapes", "Set whether fill shapes or not.", "0");
AddAction(5, af_none, "Set whether fill shapes", "", "Set whether fill shapes or not.", "Set whether fill shapes or not.", "SetFilled");

AddStringParam("Set font family", "Set the current font family.", '"Arial"');
AddAction(6, af_none, "Set font family", "", "Set the current font family.", "Set the current font family.", "SetFontFamily");

AddNumberParam("Set font size", "Set the current font size.", "12");
AddAction(7, af_none, "Set font size", "", "Set the current font size.", "Set the current font size.", "SetFontSize");

////////////////////////////////////////
// Expressions

// AddExpression(id,			// any positive integer to uniquely identify this expression
//				 flags,			// (see docs) ef_none, ef_deprecated, ef_return_number, ef_return_string,
//								// ef_return_any, ef_variadic_parameters (one return flag must be specified)
//				 list_name,		// currently ignored, but set as if appeared in event wizard
//				 category,		// category in expressions panel
//				 exp_name,		// the expression name after the dot, e.g. "foo" for "myobject.foo" - also the runtime function name
//				 description);	// description in expressions panel

// example
//AddExpression(0, ef_return_number, "Leet expression", "My category", "MyExpression", "Return the number 1337.");

AddExpression(0, ef_return_string,	"Get selected tool", "Painter Tools", "SelectedTool", "Get the tool which was selected by the user.");
AddExpression(1, ef_return_number,	"Get width of lines", "Painter Tools", "WidthOfLines", "Get the specified width of lines.");
AddExpression(2, ef_return_string,	"Get color of lines", "Painter Tools", "ColorOfLines", "Get the selected color of lines.");
AddExpression(3, ef_return_string,	"Get color of background", "Painter Tools", "ColorOfBackground", "Get the selected color of background.");
AddExpression(4, ef_return_number,	"Is filled", "Painter Tools", "IsFilled", "Return the boolean value which is true if isFilled checkbox is checked.");
AddExpression(5, ef_return_string,	"Font family", "Painter Tools", "FontFamily", "Get the current font family.");
AddExpression(6, ef_return_string,	"Font size", "Painter Tools", "FontSize", "Get size of the current font.");
AddExpression(7, ef_return_number,	"Font parameters", "Painter Tools", "FontParameters", "Get parameters of the current font.");
////////////////////////////////////////
ACESDone();

////////////////////////////////////////
// Array of property grid properties for this plugin
// new cr.Property(ept_integer,		name,	initial_value,	description)		// an integer value
// new cr.Property(ept_float,		name,	initial_value,	description)		// a float value
// new cr.Property(ept_text,		name,	initial_value,	description)		// a string
// new cr.Property(ept_color,		name,	initial_value,	description)		// a color dropdown
// new cr.Property(ept_font,		name,	"Arial,-16", 	description)		// a font with the given face name and size
// new cr.Property(ept_combo,		name,	"Item 1",		description, "Item 1|Item 2|Item 3")	// a dropdown list (initial_value is string of initially selected item)
// new cr.Property(ept_link,		name,	link_text,		description, "firstonly")		// has no associated value; simply calls "OnPropertyChanged" on click

var property_list = [
	//new cr.Property(ept_integer, 	"My property",		77,		"An example property.")
	new cr.Property(ept_combo,	"Initial visibility",	"Visible",	"Choose whether the toolbar is visible on startup.", "Invisible|Visible"),
	new cr.Property(ept_combo,	"Language",	"English",	"Language of the plugin", "English|Russian"),
	new cr.Property(ept_color, "Color of toolbar", cr.RGB(210, 105, 30), "The color of the toolbar."),
	new cr.Property(ept_combo, 	"Type of shapes",		"Pencil",		"The type of shapes.", "Pencil|Line|Rectangle|Circle|Arrow|Text|Note"),
	new cr.Property(ept_integer, 	"Width of lines",		1,		"The width of lines."),
	new cr.Property(ept_integer, 	"Fill",		0,		"Set whether fill shapes or not."),
	new cr.Property(ept_color, 	"Color of lines",		cr.RGB(0, 0, 0),		"The color of lines."),
	new cr.Property(ept_color, 	"Color of background",		cr.RGB(255, 255, 255),		"The color of the background."),
	new cr.Property(ept_font, 	"Font",		"Arial,14",		"The font of text.")
	];
	
// Called by IDE when a new object type is to be created
function CreateIDEObjectType()
{
	return new IDEObjectType();
}

// Class representing an object type in the IDE
function IDEObjectType()
{
	assert2(this instanceof arguments.callee, "Constructor called as a function");
}

// Called by IDE when a new object instance of this type is to be created
IDEObjectType.prototype.CreateInstance = function(instance)
{
	return new IDEInstance(instance);
}

// Class representing an individual instance of an object in the IDE
function IDEInstance(instance, type)
{
	assert2(this instanceof arguments.callee, "Constructor called as a function");
	
	// Save the constructor parameters
	this.instance = instance;
	this.type = type;
	
	// Set the default property values from the property table
	this.properties = {};
	
	for (var i = 0; i < property_list.length; i++)
		this.properties[property_list[i].name] = property_list[i].initial_value;
		
	// Plugin-specific variables
	// this.myValue = 0...
}

// Called when inserted via Insert Object Dialog for the first time
IDEInstance.prototype.OnInserted = function()
{
	this.instance.SetSize(new cr.vector2(185, 200));
}

// Called when double clicked in layout
IDEInstance.prototype.OnDoubleClicked = function()
{
}

// Called after a property has been changed in the properties bar
IDEInstance.prototype.OnPropertyChanged = function(property_name)
{
}

// For rendered objects to load fonts or textures
IDEInstance.prototype.OnRendererInit = function(renderer)
{
}

// Called to draw self in the editor if a layout object
IDEInstance.prototype.Draw = function(renderer)
{
	renderer.SetTexture(null);
	var quad = this.instance.GetBoundingQuad();
	
	renderer.Fill(quad, cr.RGB(255, 185, 110));
	renderer.Outline(quad, cr.RGB(0, 0, 0));
}

// For rendered objects to release fonts or textures
IDEInstance.prototype.OnRendererReleased = function(renderer)
{
}